/*** -*- Mode: Javascript; tab-width: 2;
  
The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib team code.  
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.
  
Original Author: Pete Collins <pete@mozdev.org>
Contributor(s):
    
***/


if (typeof(JS_LIB_LOADED)=='boolean') 
{

  const JS_PREFS_LOADED   = true;
  const JS_PREFS_FILE     = 'prefs.js';
  const JS_PREFS_CID      = "@mozilla.org/preferences;1";
  const JS_PREFS_I_PREF   = "nsIPref";

  /*********** PREFS *******************/
  function Prefs () 
  {
    try {
      // create instance of prefs xpcom object
      this.prefInst = jslibCreateInstance(JS_PREFS_CID, JS_PREFS_I_PREF);
      this.prefInst = jslibQI(this.prefInst, "nsIPrefBranch");
    } catch (e) { jslibError(e); }
  }

  Prefs.prototype = 
  {
    prefInst: null,

  /*********** GET Type ****************/
  // pref type
  getType : function (aPrefString) 
  {
    var rv;
    try {
      rv = this.prefInst.GetPrefType(aPrefString)
    } catch (e) {
      jslibError(e);
      rv=undefined;
    }
    return rv;
  },
  /*********** GET Type ****************/

  /*********** SET PREF ****************/
  // bool
  setBool : function (aPrefString, aInBool) 
  {
    if (!aPrefString)
      return false;

    var rv;
    try {
      this.prefInst.SetBoolPref(aPrefString, aInBool)
      rv = jslib_results.NS_OK;

    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET PREF ****************/

  /*********** GET PREF ****************/
  // bool
  getBool : function (aPrefString) 
  {
    if (!aPrefString)
      return false;

    var rv;
    try {
      rv = this.prefInst.GetBoolPref(aPrefString)
    } catch(e) {
    	rv=false;
    }
    return rv;
  },
  /*********** GET PREF ****************/

  /*********** SET CHAR PREF ***********/
  // nsresult
  setChar : function (aPrefName, aPrefString) 
  {
    if (!aPrefName && !aPrefString)
      throw "jslib:setChar not enough args provided\n";

    var rv;
    try {
      this.prefInst.SetCharPref(aPrefName, aPrefString)
      rv = jslib_results.NS_OK;

    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET CHAR PREF ***********/

  /*********** GET CHAR PREF ***********/
  // string
  getChar : function (aPrefName) 
  {
    if (!aPrefName)
      throw "jslib:getChar not enough args provided\n";

    var rv = "";
    try {
      rv = this.prefInst.GetCharPref(aPrefName)
    } catch(e) {
    	rv="";
    }
    return rv;
  },
  /*********** SET CHAR PREF ***********/

  /*********** SET INT PREF ***********/
  // nsresult
  setInt : function (aPrefName, aInt) 
  {
    if (!aPrefName && !aInt)
      throw "jslib:setInt not enough args provided\n";

    var rv;
    try {
      this.prefInst.SetIntPref(aPrefName, aInt)
      rv = C.results.NS_OK;

    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET INT PREF ***********/

  /*********** GET INT PREF ***********/
  // int
  getInt : function (aPrefName) 
  {
    if (!aPrefName)
      throw "jslib:getInt not enough args provided\n";

    var rv;
    try {
      rv = this.prefInst.GetIntPref(aPrefName)
    } catch(e) {
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SET INT PREF ***********/

  /*********** RESET PREF *************/
  // nsresult
  reset : function () 
  {
    var rv;
    try {
      this.prefInst.ResetPrefs();
      rv = jslib_results.NS_OK;
    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** RESET PREF *************/

  /*********** RESET USER PREF ********/
  // nsresult
  resetUser : function () 
  {
    var rv;
    try {
      this.prefInst.ResetUserPrefs();
      rv = jslib_results.NS_OK;
    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** RESET USER PREF ********/

  /*********** SAVE PREF **************/
  // nsresult
  save : function (aFile) 
  {
    var file;
    switch (typeof(aFile)) 
    {
      case "object":
        // check object is an nsIFile object
        if (typeof(aFile.path) == "string")
        file = aFile;
        break;

      case "string":
        include (jslib_file);
        // path is a string, make it into an nsIFile
        file = (new File(aFile)).nsIFile;
        break;
    }

    var rv;
    try {
      this.prefInst.savePrefFile(file);
      rv = jslib_results.NS_OK;
    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** SAVE PREF **************/

  /*********** CLEAR USER PREF ********/
  // nsresult
  clear : function (aPrefString) 
  {
    var rv;
    try {
      this.prefInst.ClearUserPref(aPrefString);
      rv = jslib_results.NS_OK;
    } catch(e) {
			jslibError(e);
    	rv=-jslib_results.NS_ERROR_FAILURE;
    }
    return rv;
  },
  /*********** CLEAR USER PREF ********/

  /*********** GET NSIPREF ********/
  get nsIPref () { return this.prefInst; }
  /*********** GET NSIPREF ********/

  } // END CLASS
  /*********** PREFS *******************/

  jslibDebug('*** load: '+JS_PREFS_FILE+' OK');

} // END BLOCK JS_LOADED CHECK
    
// If jslib base library is not loaded, dump this error.
else
{  
    dump("JS_PREFS library not loaded:\n"                           +
         " \tTo load use: chrome://jslib/content/jslib.js\n"        +
         " \tThen: include (jslib_prefs);\n\n");
}
